const Fragment = require('./Fragment')
const User = require('./User')
const get = require('lodash/get')
const config = require('config')
const mailService = require('pubsweet-component-mail-service')

const manuscriptTypes = config.get('manuscript-types')

class Email {
  constructor({
    baseUrl,
    authors = {},
    UserModel = {},
    collection = {},
    parsedFragment = {},
  }) {
    this.baseUrl = baseUrl
    this.authors = authors
    this.UserModel = UserModel
    this.collection = collection
    this.parsedFragment = parsedFragment
  }

  set _fragment(newFragment) {
    this.parsedFragment = newFragment
  }

  async setupReviewersEmail({
    FragmentModel,
    agree = false,
    isSubmitted = false,
    recommendation = {},
  }) {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { recommendations, title, type },
      authors: { submittingAuthor: { firstName = '', lastName = '' } },
    } = this
    const { parsedFragment: { id } } = this
    const fragment = await FragmentModel.find(id)
    const fragmentHelper = new Fragment({ fragment })
    const reviewerInvitations = fragmentHelper.getReviewerInvitations({
      agree,
    })
    const hasReview = invUserId => rec =>
      rec.recommendationType === 'review' &&
      rec.submittedOn &&
      invUserId === rec.userId
    const reviewerPromises = await reviewerInvitations.map(async inv => {
      if (!agree) return UserModel.find(inv.userId)
      const submittedReview = recommendations.find(hasReview(inv.userId))
      const shouldReturnUser =
        (isSubmitted && submittedReview) || (!isSubmitted && !submittedReview)
      if (shouldReturnUser) return UserModel.find(inv.userId)
    })

    let emailType = 'agreed-reviewers-after-recommendation'
    let emailText, subject, manuscriptType

    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()
    const editorName = isSubmitted
      ? `${eic.firstName} ${eic.lastName}`
      : collection.handlingEditor.name

    let reviewers = await Promise.all(reviewerPromises)
    reviewers = reviewers.filter(Boolean)

    if (agree) {
      subject = isSubmitted
        ? `${collection.customId}: Manuscript Decision`
        : `${collection.customId}: Manuscript ${getSubject(recommendation)}`

      if (isSubmitted) {
        emailType = 'submitting-reviewers-after-decision'
        emailText = 'has now been rejected'
        if (recommendation === 'publish') emailText = 'will now be published'
      }
    } else {
      subject = `${collection.customId}: Reviewer Unassigned`
      manuscriptType = manuscriptTypes[type]
      emailType = 'no-response-reviewers-after-recommendation'
    }

    reviewers.forEach(user =>
      mailService.sendNotificationEmail({
        emailType,
        toEmail: user.email,
        meta: {
          baseUrl,
          emailText,
          editorName,
          collection,
          manuscriptType,
          timestamp: Date.now(),
          emailSubject: subject,
          reviewerName: `${user.firstName} ${user.lastName}`,
          fragment: {
            title,
            authorName: `${firstName} ${lastName}`,
            id,
          },
        },
      }),
    )
  }

  async sendNewVersionSubmittedReviewersEmail({
    newFragmentId = '',
    invitations = [],
  }) {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { title },
      authors: { submittingAuthor: { firstName = '', lastName = '' } },
    } = this
    const reviewers = await Promise.all(
      invitations.map(async inv => UserModel.find(inv.userId)),
    )
    reviewers.forEach(user =>
      mailService.sendNotificationEmail({
        emailType: 'submitting-reviewers-after-revision',
        toEmail: user.email,
        meta: {
          baseUrl,
          collection,
          timestamp: Date.now(),
          reviewerName: `${user.firstName} ${user.lastName}`,
          fragment: {
            title,
            authorName: `${firstName} ${lastName}`,
            id: newFragmentId,
          },
        },
      }),
    )
  }

  async setupAuthorsEmail({
    requestToRevision = false,
    publish = false,
    FragmentModel,
  }) {
    const {
      baseUrl,
      collection,
      parsedFragment: { heRecommendation, id, title, newComments },
      authors: { submittingAuthor: { email, firstName, lastName } },
    } = this
    let comments = get(heRecommendation, 'comments') || []
    if (requestToRevision) comments = newComments
    const authorNote = comments.find(comm => comm.public)
    const content = get(authorNote, 'content')
    const authorNoteText = content ? `Reason & Details: "${content}"` : ''
    let emailType = requestToRevision
      ? 'author-request-to-revision'
      : 'author-manuscript-rejected'
    if (publish) emailType = 'author-manuscript-published'
    let toAuthors = null
    if (emailType === 'author-request-to-revision') {
      toAuthors = [
        {
          email,
          name: `${firstName} ${lastName}`,
        },
      ]
    } else {
      const fragment = await FragmentModel.find(id)
      toAuthors = fragment.authors.map(author => ({
        email: author.email,
        name: `${author.firstName} ${author.lastName}`,
      }))
    }
    toAuthors.forEach(toAuthor => {
      mailService.sendNotificationEmail({
        emailType,
        toEmail: toAuthor.email,
        meta: {
          handlingEditorName: get(collection, 'handlingEditor.name'),
          baseUrl,
          collection,
          authorNoteText,
          fragment: {
            id,
            title,
            authorName: toAuthor.name,
            submittingAuthorName: `${firstName} ${lastName}`,
          },
        },
      })
    })
  }

  async setupHandlingEditorEmail({
    publish = false,
    returnWithComments = false,
    reviewSubmitted = false,
    reviewerName = '',
  }) {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { eicComments = '', title, id },
      authors: { submittingAuthor: { firstName = '', lastName = '' } },
    } = this
    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()
    const toEmail = get(collection, 'handlingEditor.email')
    let emailType = publish
      ? 'he-manuscript-published'
      : 'he-manuscript-rejected'
    if (reviewSubmitted) emailType = 'review-submitted'
    if (returnWithComments) emailType = 'he-manuscript-return-with-comments'
    mailService.sendNotificationEmail({
      toEmail,
      emailType,
      meta: {
        baseUrl,
        collection,
        reviewerName,
        eicComments,
        eicName: `${eic.firstName} ${eic.lastName}`,
        emailSubject: `${collection.customId}: Manuscript Decision`,
        handlingEditorName: get(collection, 'handlingEditor.name') || '',
        fragment: {
          id,
          title,
          authorName: `${firstName} ${lastName}`,
        },
      },
    })
  }

  async setupEiCEmail({ recommendation, comments }) {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { title, id },
      authors: { submittingAuthor: { firstName, lastName } },
    } = this
    const privateNote = comments.find(comm => comm.public === false)
    const content = get(privateNote, 'content')
    const privateNoteText =
      content !== undefined ? `Private note: "${content}"` : ''
    let paragraph
    const heRecommendation = getHeRecommendation(recommendation)
    const manuscriptAuthorText = `the manuscript titled "${title}" by ${firstName} ${lastName}`
    const publishOrRejectText =
      'It is my recommendation, based on the reviews I have received for'
    switch (heRecommendation) {
      case 'publish':
        paragraph = `${publishOrRejectText} ${manuscriptAuthorText}, that we should proceed to publication. <br/><br/>
        ${privateNoteText}<br/><br/>`
        break
      case 'reject':
        paragraph = `${publishOrRejectText}
        ${manuscriptAuthorText}, that we should reject it for publication. <br/><br/>
        ${privateNoteText}<br/><br/>`
        break
      case 'revision':
        paragraph = `In order for ${manuscriptAuthorText} to proceed to publication, there needs to be a revision. <br/><br/>
        ${privateNoteText}<br/><br/>`
        break

      default:
        throw new Error('undefined HE recommentation type')
    }

    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()
    const toEmail = eic.email
    mailService.sendNotificationEmail({
      toEmail,
      emailType: 'eic-recommendation',
      meta: {
        baseUrl,
        paragraph,
        collection,
        fragment: { id },
        eicName: `${eic.firstName} ${eic.lastName}`,
        handlingEditorName: get(collection, 'handlingEditor.name'),
      },
    })
  }

  setupReviewerInvitationEmail({
    user,
    invitationId,
    timestamp,
    resend = false,
    authorName,
  }) {
    const {
      baseUrl,
      collection,
      parsedFragment: { id, title, abstract },
      authors,
    } = this
    const params = {
      user,
      baseUrl,
      subject: `${collection.customId}: Review Requested`,
      toEmail: user.email,
      meta: {
        fragment: {
          id,
          title,
          abstract,
          authors,
        },
        invitation: {
          id: invitationId,
          timestamp,
        },
        collection: {
          id: collection.id,
          authorName,
          handlingEditor: collection.handlingEditor,
        },
      },
      emailType: resend ? 'resend-reviewer' : 'invite-reviewer',
    }
    mailService.sendReviewerInvitationEmail(params)
  }

  async setupReviewerDecisionEmail({
    user,
    agree = false,
    timestamp = Date.now(),
    authorName = '',
  }) {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { id, title },
    } = this
    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()
    const toEmail = collection.handlingEditor.email
    mailService.sendNotificationEmail({
      toEmail,
      user,
      emailType: agree ? 'reviewer-agreed' : 'reviewer-declined',
      meta: {
        collection: { customId: collection.customId, id: collection.id },
        fragment: { id, title, authorName },
        handlingEditorName: collection.handlingEditor.name,
        baseUrl,
        eicName: `${eic.firstName} ${eic.lastName}`,
        timestamp,
      },
    })
    if (agree)
      mailService.sendNotificationEmail({
        toEmail: user.email,
        user,
        emailType: 'reviewer-thank-you',
        meta: {
          collection: { customId: collection.customId, id: collection.id },
          fragment: { id, title, authorName },
          handlingEditorName: collection.handlingEditor.name,
          baseUrl,
        },
      })
  }

  async setupReviewerUnassignEmail({ user, authorName }) {
    const { collection, parsedFragment: { title = '' } } = this

    await mailService.sendNotificationEmail({
      toEmail: user.email,
      user,
      emailType: 'unassign-reviewer',
      meta: {
        collection: { customId: collection.customId },
        fragment: { title, authorName },
        handlingEditorName: collection.handlingEditor.name,
      },
    })
  }

  async setupManuscriptSubmittedEmail() {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { id, title },
      authors: { submittingAuthor: { firstName = '', lastName = '' } },
    } = this

    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()

    mailService.sendSimpleEmail({
      toEmail: eic.email,
      emailType: 'manuscript-submitted',
      dashboardUrl: baseUrl,
      meta: {
        collection,
        fragment: { id, authorName: `${firstName} ${lastName}`, title },
        eicName: `${eic.firstName} ${eic.lastName}`,
      },
    })
  }

  async setupNewVersionSubmittedEmail() {
    const {
      baseUrl,
      UserModel,
      collection,
      parsedFragment: { id, title },
      authors: { submittingAuthor: { firstName = '', lastName = '' } },
    } = this

    const userHelper = new User({ UserModel })
    const eic = await userHelper.getEditorInChief()

    mailService.sendNotificationEmail({
      toEmail: collection.handlingEditor.email,
      emailType: 'new-version-submitted',
      meta: {
        baseUrl,
        collection,
        eicName: `${eic.firstName} ${eic.lastName}`,
        fragment: { id, authorName: `${firstName} ${lastName}`, title },
        handlingEditorName: collection.handlingEditor.name,
      },
    })
  }
}

const getSubject = recommendation =>
  ['minor', 'major'].includes(recommendation)
    ? 'Revision Requested'
    : 'Recommendation Submitted'

const getHeRecommendation = recommendation => {
  let heRecommendation = recommendation === 'reject' ? 'reject' : 'publish'
  if (['minor', 'major'].includes(recommendation)) {
    heRecommendation = 'revision'
  }
  return heRecommendation
}

module.exports = Email
