require('dotenv').config()
const path = require('path')
const components = require('./components.json')
const logger = require('winston')

// const environment = process.env.NODE_ENV || 'development'

const getDbConfig = () => {
  if (process.env.DATABASE) {
    return {
      user: process.env.DB_USER,
      password: process.env.DB_PASS,
      database: process.env.DATABASE,
      host: process.env.DB_HOST,
      port: 5432,
      ssl: true,
    }
  }
  return {}
}

module.exports = {
  authsome: {
    mode: path.resolve(__dirname, 'authsome-mode.js'),
    teams: {
      handlingEditor: {
        name: 'Handling Editors',
      },
      reviewer: {
        name: 'Reviewer',
      },
    },
  },
  validations: path.resolve(__dirname, 'validations.js'),
  pubsweet: {
    components,
  },
  'pubsweet-server': {
    db: getDbConfig(),
    port: 3000,
    logger,
    uploads: 'uploads',
    secret: 'SECRET',
  },
  'pubsweet-client': {
    API_ENDPOINT: '/api',
    baseUrl: process.env.CLIENT_BASE_URL || 'http://localhost:3000',
    'login-redirect': '/',
    'redux-log': false,
    theme: process.env.PUBSWEET_THEME,
  },
  orcid: {
    clientID: process.env.ORCID_CLIENT_ID,
    clientSecret: process.env.ORCID_CLIENT_SECRET,
    callbackPath: '/api/users/orcid/callback',
    successPath: '/profile',
  },
  'mail-transport': {
    sendmail: true,
  },
  'password-reset': {
    url:
      process.env.PUBSWEET_PASSWORD_RESET_URL ||
      'http://localhost:3000/password-reset',
    sender: process.env.PUBSWEET_PASSWORD_RESET_SENDER || 'dev@example.com',
  },
  'pubsweet-component-aws-s3': {
    secretAccessKey: process.env.AWS_S3_SECRET_KEY,
    accessKeyId: process.env.AWS_S3_ACCESS_KEY,
    region: process.env.AWS_S3_REGION,
    bucket: process.env.AWS_S3_BUCKET,
    validations: path.resolve(__dirname, 'upload-validations.js'),
  },
  'invite-reset-password': {
    url: process.env.PUBSWEET_INVITE_PASSWORD_RESET_URL || '/invite',
  },
  'forgot-password': {
    url: process.env.PUBSWEET_FORGOT_PASSWORD_URL || '/forgot-password',
  },
  'invite-reviewer': {
    url: process.env.PUBSWEET_INVITE_REVIEWER_URL || '/invite-reviewer',
  },
  'confirm-signup': {
    url: process.env.PUBSWEET_CONFIRM_SIGNUP_URL || '/confirm-signup',
  },
  roles: {
    global: ['admin', 'editorInChief', 'author', 'handlingEditor'],
    collection: ['handlingEditor', 'reviewer', 'author'],
    inviteRights: {
      admin: ['admin', 'editorInChief', 'author', 'handlingEditor', 'author'],
      editorInChief: ['handlingEditor'],
      handlingEditor: ['reviewer'],
    },
  },
  mailer: {
    from: 'faraday@hindawi.com',
    path: `${__dirname}/mailer`,
  },
  publicKeys: [
    'pubsweet-client',
    'authsome',
    'validations',
    'pubsweet-component-aws-s3',
    'pubsweet-component-aws-ses',
  ],
  statuses: {
    draft: {
      importance: 1,
      author: {
        label: 'Complete Submission',
        needsAttention: true,
      },
      admin: {
        label: 'Complete Submission',
        needsAttention: true,
      },
    },
    technicalChecks: {
      importance: 2,
      author: {
        label: 'Submitted',
        needsAttention: false,
      },
      editorInChief: {
        label: 'QA',
        needsAttention: false,
      },
      admin: {
        label: 'Approve QA',
        needsAttention: true,
      },
    },
    submitted: {
      importance: 3,
      author: {
        label: 'Submitted',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Assign HE',
        needsAttention: true,
      },
      admin: {
        label: 'Assign HE',
        needsAttention: true,
      },
    },
    heInvited: {
      importance: 4,
      author: {
        label: 'HE Invited',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Respond to Invite',
        needsAttention: true,
      },
      editorInChief: {
        label: 'HE Invited',
        needsAttention: false,
      },
      admin: {
        label: 'Respond to Invite',
        needsAttention: true,
      },
    },
    heAssigned: {
      importance: 5,
      author: {
        label: 'HE Assigned',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Invite Reviewers',
        needsAttention: true,
      },
      editorInChief: {
        label: 'HE Assigned',
        needsAttention: false,
      },
      admin: {
        label: 'Invite Reviewers',
        needsAttention: true,
      },
    },
    reviewersInvited: {
      importance: 6,
      author: {
        label: 'Reviewers Invited',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Check Review Process',
        needsAttention: true,
      },
      editorInChief: {
        label: 'Reviewers Invited',
        needsAttention: false,
      },
      reviewer: {
        label: 'Respond to Invite',
        needsAttention: true,
      },
      admin: {
        label: 'Respond to Invite',
        needsAttention: true,
      },
    },
    underReview: {
      importance: 7,
      author: {
        label: 'Under Review',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Check Review Process',
        needsAttention: true,
      },
      editorInChief: {
        label: 'Under Review',
        needsAttention: false,
      },
      reviewer: {
        label: 'Complete Review',
        needsAttention: true,
      },
      admin: {
        label: 'Complete Review',
        needsAttention: true,
      },
    },
    reviewCompleted: {
      importance: 8,
      author: {
        label: 'Review Completed',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Make Recommendation',
        needsAttention: true,
      },
      editorInChief: {
        label: 'Review Completed',
        needsAttention: false,
      },
      reviewer: {
        label: 'Review Completed',
        needsAttention: false,
      },
      admin: {
        label: 'Make Recommendation',
        needsAttention: true,
      },
    },
    revisionRequested: {
      importance: 9,
      author: {
        label: 'Submit Revision',
        needsAttention: true,
      },
      handlingEditor: {
        label: 'Revision Requested',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Revision Requested',
        needsAttention: false,
      },
      reviewer: {
        label: 'Revision Requested',
        needsAttention: false,
      },
      admin: {
        label: 'Submit Revision',
        needsAttention: true,
      },
    },
    pendingApproval: {
      importance: 10,
      author: {
        label: 'Pending Approval',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Pending Approval',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Make Decision',
        needsAttention: true,
      },
      reviewer: {
        label: 'Pending Approval',
        needsAttention: false,
      },
      admin: {
        label: 'Make Decision',
        needsAttention: true,
      },
    },
    rejected: {
      importance: 11,
      author: {
        label: 'Rejected',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Rejected',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Rejected',
        needsAttention: false,
      },
      reviewer: {
        label: 'Rejected',
        needsAttention: false,
      },
      admin: {
        label: 'Rejected',
        needsAttention: false,
      },
    },
    inQA: {
      importance: 12,
      author: {
        label: 'Pending approval',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'QA',
        needsAttention: false,
      },
      editorInChief: {
        label: 'QA',
        needsAttention: false,
      },
      reviewer: {
        label: 'QA',
        needsAttention: false,
      },
      admin: {
        label: 'Approve QA',
        needsAttention: true,
      },
    },
    accepted: {
      importance: 13,
      author: {
        label: 'Accepted',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Accepted',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Accepted',
        needsAttention: false,
      },
      reviewer: {
        label: 'Accepted',
        needsAttention: false,
      },
      admin: {
        label: 'Accepted',
        needsAttention: false,
      },
    },
    withdrawalRequested: {
      importance: 14,
      author: {
        label: 'Withdrawal Requested',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Withdrawal Requested',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Approve Withdrawal',
        needsAttention: true,
      },
      reviewer: {
        label: 'Withdrawal Requested',
        needsAttention: false,
      },
      admin: {
        label: 'Approve Withdrawal',
        needsAttention: true,
      },
    },
    withdrawn: {
      importance: 15,
      author: {
        label: 'Withdrawn',
        needsAttention: false,
      },
      handlingEditor: {
        label: 'Withdrawn',
        needsAttention: false,
      },
      editorInChief: {
        label: 'Withdrawn',
        needsAttention: false,
      },
      reviewer: {
        label: 'Withdrawn',
        needsAttention: false,
      },
      admin: {
        label: 'Withdrawn',
        needsAttention: false,
      },
    },
  },
  'manuscript-types': {
    research: 'Research',
    review: 'Review',
    'clinical-study': 'Clinical Study',
    'case-report': 'Case Report',
    'letter-to-editor': 'Letter to the Editor',
    editorial: 'Editorial',
    corrigendum: 'Corrigendum',
    erratum: 'Erratum',
    'expression-of-concern': 'Expression of Concern',
    retraction: 'Retraction',
  },
}
